#ifndef __SORTING_STRINGS_TERMINATED_BY_ZERO_MACROS_H_
#define __SORTING_STRINGS_TERMINATED_BY_ZERO_MACROS_H_


// For size_t, malloc and free.
#include <stdio.h>
#include <stdlib.h>


// Define static functions to sort an array by multi-key quick sort.
// Depth of recursive calls is limited to O(log_n).
//
// Arguments are as follows:
// - StringSort: Function name, also used for the prefix of internal functions.
// - Type: Type of keys.
// - GetLetter: Function to get a letter from a key.
// - HandleSameKeys: Function to handle same keys found in sorting.
//
// Example: DEFINE_SZSORT(string_sort, key_t, get_letter, handle_same_keys);
//
// This macro defines a function string_sort() to sort an array of key_t.
//  "static void string_sort(key_t *, key_t *, size_t)"
// Two user-defined functions get_letter() and handle_same_keys() are used in
// sorting, and those functions must be defined as follows:
//  "static unsigned char get_letter(const key_t *, size_t depth)"
//  "static void handle_same_keys(key_t *, key_t *)"
#define DEFINE_SZSORT(StringSort, Type, GetLetter, HandleSameKeys)\
\
/* To avoid problems about const. */\
typedef Type StringSort ## Type;\
typedef const StringSort ## Type *StringSort ## Pointer;\
\
/* These declarations are to check the function types. */\
static unsigned char GetLetter(StringSort ## Pointer key, size_t depth);\
static void HandleSameKeys(Type *l, Type *r);\
\
/* Function to choose the median from 3 values. */\
static unsigned char StringSort ## Median(unsigned char l, unsigned char m,\
                                          unsigned char r) {\
  if (l < m) {\
    if (m < r) return m;\
    else if (l < r) return r;\
    return l;\
  } else if (l < r) return l;\
  else if (m < r) return r;\
  return m;\
}\
\
/* Swap a pair of keys. */\
static void StringSort ## SwapKeys(Type *lhs, Type *rhs) {\
  Type tmp = *lhs;\
  *lhs = *rhs;\
  *rhs = tmp;\
}\
\
/* Compare a pair of keys. */\
static int StringSort ## CompareKeys(\
    StringSort ## Pointer lhs, StringSort ## Pointer rhs, size_t depth) {\
  unsigned char l, r;\
  do {\
    l = GetLetter(lhs, depth);\
    r = GetLetter(rhs, depth++);\
  } while (l != '\0' && l == r);\
  return l - r;\
}\
\
/* Find same keys and call HandleSameKeys() for them. */\
static void StringSort ## FindSameKeys(Type *l, Type *r, size_t depth) {\
  Type *start = l;\
  for (l++; l < r; l++) {\
    if (StringSort ## CompareKeys(start, l, depth)) {\
      if (l - start > 1)\
        HandleSameKeys(start, l);\
      start = l;\
    }\
  }\
  if (r - start > 1) HandleSameKeys(start, r);\
}\
\
/* Insertion sort for small range. */\
static void StringSort ## InsertionSort(Type *l, Type *r, size_t depth) {\
  int equal_flag = 0;\
  Type *i, *j;\
  for (i = l + 1; i < r; i++) {\
    Type key = *i;\
    for (j = i; j > l; j--) {\
      int compare = StringSort ## CompareKeys(&key, j - 1, depth);\
      equal_flag |= (compare == 0);\
      if (compare >= 0) break;\
      *j = *(j - 1);\
    }\
    *j = key;\
  }\
  if (equal_flag) StringSort ## FindSameKeys(l, r, depth);\
}\
\
/* Sort an array by multi-key quick sort. */\
static void StringSort(Type *l, Type *r, size_t depth) {\
  while (r - l > 10) {\
    Type *pl = l, *pr = r, *pivot_l = l, *pivot_r = r;\
    unsigned char pivot;\
    pivot = StringSort ## Median(GetLetter(l, depth),\
                                 GetLetter(l + (r - l) / 2, depth),\
                                 GetLetter(r - 1, depth));\
\
    /* Move keys less than the pivot to left. */\
    /* Move keys greater than the pivot to right. */\
    for (;;) {\
      int diff;\
      while (pl < pr && (diff = GetLetter(pl, depth) - pivot) <= 0) {\
        if (!diff) StringSort ## SwapKeys(pl, pivot_l++);\
        pl++;\
      }\
      while (pl < pr && (diff = GetLetter(--pr, depth) - pivot) >= 0) {\
        if (!diff) StringSort ## SwapKeys(pr, --pivot_r);\
      }\
      if (pl >= pr) break;\
      StringSort ## SwapKeys(pl++, pr);\
    }\
    while (pivot_l > l) StringSort ## SwapKeys(--pivot_l, --pl);\
    while (pivot_r < r) StringSort ## SwapKeys(pivot_r++, pr++);\
\
    /* To avoid stack over flow. */\
    /* Use recursive call only for the smallest range. */\
    if (pl - l > pr - pl || r - pr > pr - pl) {\
      if (pr - pl > 1) {\
        if (pivot != '\0') StringSort(pl, pr, depth + 1);\
        else HandleSameKeys(pl, pr);\
      }\
      if (pl - l < r - pr) {\
        if (pl - l > 1) StringSort(l, pl, depth);\
        l = pr;\
      } else {\
        if (r - pr > 1) StringSort(pr, r, depth);\
        r = pl;\
      }\
    } else {\
      if (pl - l > 1) StringSort(l, pl, depth);\
      if (r - pr > 1) StringSort(pr, r, depth);\
      l = pl, r = pr;\
      if (pr - pl > 1) {\
        if (pivot != '\0') depth++;\
        else HandleSameKeys(pl, pr), l = r;\
      }\
    }\
  }\
  if (r - l > 1) StringSort ## InsertionSort(l, r, depth);\
}\
\
/* Declaration of the multi-key quick sort function. */\
/* Semicolon must be added after this macro because of this declaration. */\
static void StringSort(Type *l, Type *r, size_t depth)


// Define static functions to sort an array by using the combination of
// radix sort and multi-key quick sort.
// This macro is suitable for an array which has many keys.
//
// This macro needs similar arguments that DEFINE_SZSORT() needs.
// Please see the comments about DEFINE_SZSORT().
#define DEFINE_HSZSORT(HugeStringSort, Type, GetLetter, HandleSameKeys)\
\
/* Define functions for multi-key quick sort. */\
DEFINE_SZSORT(HugeStringSort ## StringSort, Type, GetLetter, HandleSameKeys);\
\
/* Calculate the bucket id from the head 2 letters. */\
static size_t HugeStringSort ## GetBucket(\
    HugeStringSort ## StringSort ## Pointer key, size_t depth) {\
  size_t id = GetLetter(key, depth);\
  return id ? ((id << 8) + GetLetter(key, depth + 1)) : 0;\
}\
\
/* Sort an array which has many keys. */\
/* The combination of radix sort and multi-key quick sort is used. */\
static Type *HugeStringSort(Type *l, Type *r, size_t depth) {\
  size_t i, num = r - l;\
  size_t *buf, *bucket;\
  Type *keys, *p;\
\
  if ((keys = (Type *)malloc(sizeof(*keys) * num)) == NULL)\
    return NULL;\
  if ((buf = (size_t *)malloc(sizeof(*buf) * 65538)) == NULL) {\
    free(keys);\
    return NULL;\
  }\
\
  /* Divide keys to buckets by using their head 2 letters. */\
  for (bucket = buf + 2, i = 0; i < 65538; i++)\
    buf[i] = 0;\
  for (p = l; p < r; p++)\
    bucket[HugeStringSort ## GetBucket(p, depth)]++;\
  for (bucket--, i = 2; i < 65536; i++)\
    bucket[i] += bucket[i - 1];\
  for (p = l; p < r; p++)\
    keys[bucket[HugeStringSort ## GetBucket(p, depth)]++] = *p;\
\
  /* Apply multi-key quick sort to each bucket. */\
  for (bucket--, i = 0; i < 65536; ++i) {\
    if (bucket[i + 1] - bucket[i] > 1) {\
      if (i & 0xFF)\
        HugeStringSort ## StringSort(keys + bucket[i],\
                                     keys + bucket[i + 1], depth + 2);\
      else\
        HandleSameKeys(keys + bucket[i], keys + bucket[i + 1]);\
    }\
  }\
\
  free(buf);\
  return keys;\
}\
\
/* Declaration of the radix and multi-key quick sort function. */\
/* Semicolon must be added after this macro because of this declaration. */\
static Type *HugeStringSort(Type *l, Type *r, size_t depth)


#endif  // __SORTING_STRINGS_TERMINATED_BY_ZERO_MACROS_H_
